﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  internal class ComponentReducedLinearModel : RegressionModel, IConfidenceRegressionModel {
    [Storable]
    private IConfidenceRegressionModel Model;
    [Storable]
    private PrincipleComponentAnalysisStatic Pca;

    [StorableConstructor]
    private ComponentReducedLinearModel(bool deserializing) : base(deserializing) { }
    private ComponentReducedLinearModel(ComponentReducedLinearModel original, Cloner cloner) : base(original, cloner) {
      Model = cloner.Clone(original.Model);
      Pca = cloner.Clone(original.Pca);
    }
    public ComponentReducedLinearModel(string targetVariable, IConfidenceRegressionModel model, PrincipleComponentAnalysisStatic pca) : base(targetVariable) {
      Model = model;
      Pca = pca;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ComponentReducedLinearModel(this, cloner);
    }

    public override IEnumerable<string> VariablesUsedForPrediction {
      get { return Model.VariablesUsedForPrediction; }
    }
    public override IEnumerable<double> GetEstimatedValues(IDataset dataset, IEnumerable<int> rows) {
      return Model.GetEstimatedValues(Pca.ProjectDataset(dataset), rows);
    }
    public override IRegressionSolution CreateRegressionSolution(IRegressionProblemData problemData) {
      return new ConfidenceRegressionSolution(this, problemData);
    }
    public IEnumerable<double> GetEstimatedVariances(IDataset dataset, IEnumerable<int> rows) {
      return Model.GetEstimatedVariances(Pca.ProjectDataset(dataset), rows);
    }
  }
}