﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("NoPruning", "No pruning")]
  public class NoPruning : PruningBase {
    #region Constructors & Cloning
    [StorableConstructor]
    private NoPruning(bool deserializing) : base(deserializing) { }
    private NoPruning(NoPruning original, Cloner cloner) : base(original, cloner) { }
    public NoPruning() : base() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NoPruning(this, cloner);
    }
    #endregion

    #region IPruningType
    public override ILeafType<IRegressionModel> ModelType(ILeafType<IRegressionModel> leafType) {
      return null;
    }

    public override void GenerateHoldOutSet(IReadOnlyList<int> allrows, IRandom random, out IReadOnlyList<int> training, out IReadOnlyList<int> holdout) {
      training = allrows;
      holdout = allrows;
    }
    internal override bool Prune(M5NodeModel node, M5CreationParameters m5CreationParams, IReadOnlyList<int> testRows, double globalStdDev) {
      return false;
    }
    #endregion
  }
}