﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("OrderImpurityType", "A split selector that uses the ratio between Variances^1/Order to determine good splits")]
  public class OrderImpurityType : ParameterizedNamedItem, IImpurityType {
    public const string OrderParameterName = "Order";
    public IFixedValueParameter<DoubleValue> OrderParameter {
      get { return Parameters[OrderParameterName] as IFixedValueParameter<DoubleValue>; }
    }
    public double Order {
      get { return OrderParameter.Value.Value; }
    }

    #region Constructors & Cloning
    [StorableConstructor]
    private OrderImpurityType(bool deserializing) { }
    private OrderImpurityType(OrderImpurityType original, Cloner cloner) : base(original, cloner) { }
    public OrderImpurityType() {
      Parameters.Add(new FixedValueParameter<DoubleValue>(OrderParameterName, "The exponent in the impurity calculation sum (x_i - x_avg)^Order.", new DoubleValue(4)));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new OrderImpurityType(this, cloner);
    }
    #endregion

    #region IImpurityType
    public IImpurityCalculator GenerateCalculator(int partition, IReadOnlyCollection<double> data) {
      return new OrderImpurityCalculator(partition, data, Order);
    }
    #endregion
  }
}